package in.barc.iitb.cfdvs.SPLANEModel.transformations;

import in.barc.iitb.cfdvs.SPLANE.Reasoner.SplaneModelReasoner;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import es.us.isa.FAMA.Reasoner.Reasoner;
import es.us.isa.FAMA.models.FAMAfeatureModel.Dependency;
import es.us.isa.FAMA.models.FAMAfeatureModel.ExcludesDependency;
import es.us.isa.FAMA.models.FAMAfeatureModel.FAMAFeatureModel;
import es.us.isa.FAMA.models.FAMAfeatureModel.Feature;
import es.us.isa.FAMA.models.FAMAfeatureModel.Relation;
import es.us.isa.FAMA.models.FAMAfeatureModel.RequiresDependency;
import es.us.isa.FAMA.models.featureModel.Cardinality;
import es.us.isa.FAMA.models.featureModel.GenericFeature;
import es.us.isa.FAMA.models.variabilityModel.TraceabilityModel;
import es.us.isa.FAMA.models.variabilityModel.VariabilityModel;
import es.us.isa.FAMA.models.variabilityModel.transformations.ISplaneTransform;

public class SplaneModelTransform implements ISplaneTransform {
	private SplaneModelReasoner r;
	private FAMAFeatureModel fm;
	private FAMAFeatureModel cm;
	private TraceabilityModel tm;
	

	public void transform(VariabilityModel featureModel, VariabilityModel componentModel,
			TraceabilityModel traceabilityModel,Reasoner reasoner) {
		this.r = (SplaneModelReasoner) reasoner;
		fm = (FAMAFeatureModel) featureModel;
		cm = (FAMAFeatureModel) componentModel;
		tm = traceabilityModel;
		r.reset();
		setSplaneModel();
	}

	private void setSplaneModel() {
		
		generateVariables(fm);
		generateConstraints(fm);
		r.setTurn(false);
		r.setNumVar(1);
		generateVariables(cm);
		generateConstraints(cm);
		generateTraceability(tm);
		r.generateComponentConstraints();
		r.generateFeatureConstraints();
		r.generateTraceabilityFormula();
	}

	private void generateTraceability(TraceabilityModel tm2) {
		Map<String,List<List<String>>> trace = tm2.getTraceabilityOrgMap();
		r.addTraceability(trace);
	}

	public void update() {

	}

	private void generateConstraints(FAMAFeatureModel fm) {
		Feature root = fm.getRoot();
		r.addRoot(root);
		generateConstraints(root);
		Iterator<Dependency> it = fm.getDependencies();
		while (it.hasNext()) {
			Dependency dep = it.next();
			if (dep instanceof RequiresDependency)
				r.addRequires(dep, dep.getOrigin(), dep.getDestination());
			else if (dep instanceof ExcludesDependency)
				r.addExcludes(dep, dep.getOrigin(), dep.getDestination());
		}
	}

	private void generateConstraints(Feature f) {
		Iterator<Relation> relations = f.getRelations();
		while (relations.hasNext()) {
			Relation rel = relations.next();
			if (rel.getNumberOfDestination() == 1) {
				if (rel.isMandatory()) {
					r.addMandatory(rel, rel.getDestinationAt(0), f);
				} else if (rel.isOptional()) {
					r.addOptional(rel, rel.getDestinationAt(0), f);
				} else {
					r.addCardinality(rel, rel.getDestinationAt(0), f, rel
							.getCardinalities());
				}
				generateConstraints(rel.getDestinationAt(0));
			} else {
				Collection<GenericFeature> children = new ArrayList<GenericFeature>();
				Iterator<Feature> it = rel.getDestination();
				while (it.hasNext()) {
					Feature child = it.next();
					children.add(child);
					generateConstraints(child);
				}
				Collection<Cardinality> cards = new ArrayList<Cardinality>();
				Iterator<Cardinality> itc = rel.getCardinalities();
				while (itc.hasNext()) {
					cards.add(itc.next());
				}
				r.addSet(rel, f, children, cards);
			}
		}
	}

	private void generateVariables(FAMAFeatureModel fm) {
		Iterator<Feature> it=null;
		if (fm.getOrderingHeuristic() != null) {
			it = fm.getOrderingHeuristic().orderFM(fm).iterator();
		} else {
			 it = fm.getFeatures().iterator();
		}
		while (it.hasNext()) {
				Feature f = (Feature) it.next();
				Relation parentRelation = f.getParent();
				if (parentRelation != null
						&& parentRelation.getNumberOfDestination() == 1
						&& parentRelation.getCardinalities().hasNext()) {
					Iterator<Cardinality> itc = parentRelation
							.getCardinalities();
					Collection<Cardinality> cards = new ArrayList<Cardinality>();
					while (itc.hasNext())
						cards.add(itc.next());
					r.addFeature(f);
				} else {
					ArrayList<Cardinality> cards = new ArrayList<Cardinality>();
					cards.add(new Cardinality(0, 1));
					r.addFeature(f);
				}
			}
		}
}
