/*
	This file is part of FaMaTS.

    FaMaTS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FaMaTS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with FaMaTS.  If not, see <http://www.gnu.org/licenses/>.

 */
package in.barc.iitb.cfdvs.QSATReasoner;

import in.barc.iitb.cfdvs.QSATConstants.QBFFormulaConstants;
import in.barc.iitb.cfdvs.QSATException.ElementNotFoundException;
import in.barc.iitb.cfdvs.QSATException.QBFFormulaGenerationException;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import es.us.isa.FAMA.Benchmarking.PerformanceResult;
import es.us.isa.FAMA.Reasoner.Question;
import es.us.isa.FAMA.Reasoner.Reasoner;

public abstract class QSATQuestion implements Question {

	protected String questionId;
	public String prepath="";
	
	abstract public void generateQBFFormula(QSATReasoner reasoner)
			throws QBFFormulaGenerationException;

	public Class<? extends Reasoner> getReasonerClass() {
		return QSATReasoner.class;
	}

	public String getQuestionId() {
		return questionId;
	}

	public void setQuestionId(String questionId) {
		this.questionId = questionId;
	}

	public QSATQuestion() {
	}

	public void preAnswer(Reasoner r) {
	}

	public PerformanceResult answer(Reasoner r) {
		return new QSATResult();
	}

	public void postAnswer(Reasoner r) {
	}

	
	protected String generateImplGlobal(Map<String, String> componentsMap,
			String featureTrace) {
		String str = "";
		str += "(" + QBFFormulaConstants.QUANTIFIER_FORALL + "[ ";
		Iterator<Map.Entry<String, String>> entries = componentsMap.entrySet()
				.iterator();
		while (entries.hasNext()) {
			Map.Entry<String, String> entry = entries.next();
			str += entry.getValue() + " ";
		}
		str += "] ( !(";
		entries = componentsMap.entrySet().iterator();
		while (entries.hasNext()) {
			Map.Entry<String, String> entry = entries.next();
			str += "(( !p" + entry.getValue() + " | " + entry.getValue()
					+ " ) & ( p" + entry.getValue() + " | !" + entry.getValue()
					+ " ))";
			if (entries.hasNext()) {
				str += " & ";
			}
		}
		str += ") | " + featureTrace + " ) )";
		return str;
	}

	protected boolean validFeatures(List<String> specification,
			Map<String, String> featuresMap) throws ElementNotFoundException {
		Iterator<String> itr = specification.iterator();
		while (itr.hasNext()) {
			String feature = itr.next();
			if (featuresMap.get(feature) == null) {
				throw new ElementNotFoundException(feature
						+ " feature is not present in given model.",
						QSATQuestion.class.toString());
			}
		}
		return true;
	}

	protected boolean validComponents(List<String> implementation,
			Map<String, String> componentsMap) throws ElementNotFoundException {
		Iterator<String> itr = implementation.iterator();
		while (itr.hasNext()) {
			String component = itr.next();
			if (componentsMap.get(component) == null) {
				throw new ElementNotFoundException(component
						+ " component is not present in given model.",
						QSATQuestion.class.toString());
			}
		}
		return true;
	}
}
