package in.barc.iitb.cfdvs.QSATReasoner;

import in.barc.iitb.cfdvs.QSATConstants.QBFFormulaConstants;
import in.barc.iitb.cfdvs.QSATException.QBFFormulaGenerationException;
import in.barc.iitb.cfdvs.SPLANE.Reasoner.SplaneModelReasoner;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;

import es.us.isa.FAMA.Benchmarking.PerformanceResult;
import es.us.isa.FAMA.Reasoner.Question;
import es.us.isa.FAMA.models.featureModel.Cardinality;
import es.us.isa.FAMA.models.featureModel.GenericFeature;
import es.us.isa.FAMA.models.featureModel.GenericRelation;
import es.us.isa.FAMA.stagedConfigManager.Configuration;

public class QSATReasoner extends SplaneModelReasoner {

	Map<String, String> featuresMap;
	Map<String, String> componentsMap;
	Map<String, String> traceability;
	List<String> featureConstraintsList;
	List<String> componentConstraintsList;
	String traceabilityFormula;
	String featureConstraints;
	String componentConstraints;

	public String getFeatureConstraints() {
		return featureConstraints;
	}

	public String getComponentConstraints() {
		return componentConstraints;
	}

	public Map<String, String> getFeaturesMap() {
		return featuresMap;
	}

	public void setFeaturesMap(Map<String, String> featuresMap) {
		this.featuresMap = featuresMap;
	}

	public Map<String, String> getComponentsMap() {
		return componentsMap;
	}

	public void setComponentsMap(Map<String, String> componentsMap) {
		this.componentsMap = componentsMap;
	}

	public Map<String, String> getTraceability() {
		return traceability;
	}

	public void setTraceability(Map<String, String> traceability) {
		this.traceability = traceability;
	}

	public List<String> getFeatureConstraintsList() {
		return featureConstraintsList;
	}

	public void setFeatureConstraintsList(List<String> featureConstraintsList) {
		this.featureConstraintsList = featureConstraintsList;
	}

	public List<String> getComponentConstraintsList() {
		return componentConstraintsList;
	}

	public void setComponentConstraintsList(
			List<String> componentConstraintsList) {
		this.componentConstraintsList = componentConstraintsList;
	}

	public void setTraceabilityFormula(String traceabilityFormula) {
		this.traceabilityFormula = traceabilityFormula;
	}

	public void setFeatureConstraints(String featureConstraints) {
		this.featureConstraints = featureConstraints;
	}

	public void setComponentConstraints(String componentConstraints) {
		this.componentConstraints = componentConstraints;
	}

	@Override
	public void reset() {
		featuresMap = new HashMap<String, String>();
		componentsMap = new HashMap<String, String>();
		traceability = new HashMap<String, String>();

		featureConstraintsList = new ArrayList<String>();
		componentConstraintsList = new ArrayList<String>();

		featureConstraints = new String();
		componentConstraints = new String();
		traceabilityFormula = new String();

		numVar = 1;
		turn = true;
	}

	@Override
	public void addFeature(GenericFeature feature) {
		if (isTurn()) {
			featuresMap.put(feature.getName(), "f" + Integer.toString(numVar));
		} else {
			componentsMap
					.put(feature.getName(), "c" + Integer.toString(numVar));
		}
		numVar++;
	}

	@Override
	public void addRoot(GenericFeature feature) {
		if (isTurn()) {
			featureConstraintsList.add("( "
					+ featuresMap.get(feature.getName()) + " )");
		} else {
			componentConstraintsList.add("( "
					+ componentsMap.get(feature.getName()) + " )");
		}
	}

	@Override
	public void addMandatory(GenericRelation rel, GenericFeature child,
			GenericFeature parent) {
		if (isTurn()) {
			String p = featuresMap.get(parent.getName());
			String q = featuresMap.get(child.getName());
			String str = "( !" + p + " |" + " " + q + " ) & ( " + p + " | !"
					+ q + " )";
			featureConstraintsList.add("( " + str + " )");
		} else {
			String p = componentsMap.get(parent.getName());
			String q = componentsMap.get(child.getName());
			String str = "( !" + p + " |" + " " + q + " ) & ( " + p + " | !"
					+ q + " )";
			componentConstraintsList.add("( " + str + " )");
		}
	}

	@Override
	public void addOptional(GenericRelation rel, GenericFeature child,
			GenericFeature parent) {
		if (isTurn()) {
			String p = featuresMap.get(parent.getName());
			String q = featuresMap.get(child.getName());
			String str = p + " |" + " !" + q;
			featureConstraintsList.add("( " + str + " )");
		} else {
			String p = componentsMap.get(parent.getName());
			String q = componentsMap.get(child.getName());
			String str = p + " |" + " !" + q;
			componentConstraintsList.add("( " + str + " )");
		}

	}

	@Override
	public void addCardinality(GenericRelation rel, GenericFeature child,
			GenericFeature parent, Iterator<Cardinality> cardinalities) {
		// TODO Auto-generated method stub

	}

	@Override
	public void addSet(GenericRelation rel, GenericFeature parent,
			Collection<GenericFeature> children,
			Collection<Cardinality> cardinalities) {
		if (isTurn()) {
			String str = generateSet(children, cardinalities, featuresMap,
					parent.getName());
			featureConstraintsList.add(str);
		} else {
			String str = generateSet(children, cardinalities, componentsMap,
					parent.getName());
			componentConstraintsList.add(str);
		}
	}

	private String generateSet(Collection<GenericFeature> children,
			Collection<Cardinality> cardinalities,
			Map<String, String> lookUpMap, String parentFeature) {
		Iterator<Cardinality> itc = cardinalities.iterator();
		Cardinality card = null;
		String str = "";
		if (itc.hasNext()) {
			card = itc.next();
			if (card.getMin() == 1 && card.getMax() > 1) {
				str = str + "( ";
				Iterator<GenericFeature> itcFeature = children.iterator();
				while (itcFeature.hasNext()) {
					GenericFeature feature = itcFeature.next();
					String fCode = lookUpMap.get(feature.getName());
					str = str + fCode;
					if (itcFeature.hasNext()) {
						str = str + " | ";
					}
				}
				str = str + " )";
				str = "(( !" + str + " | " + lookUpMap.get(parentFeature)
						+ " ) & ( !" + lookUpMap.get(parentFeature) + " | "
						+ str + "))";
			} else if (card.getMin() == 0) {
			} else if (card.getMin() == 1 && card.getMax() == 1) {
				List<String> featureList = new ArrayList<String>();
				Iterator<GenericFeature> itcFeature = children.iterator();
				while (itcFeature.hasNext()) {
					GenericFeature feature = itcFeature.next();
					String fCode = lookUpMap.get(feature.getName());
					featureList.add(fCode);
				}
				str = str + "( ";
				for (int i = 0; i < featureList.size(); i++) {
					String pos = "";
					String neg = "";
					str = str + "( ";
					for (int j = 0; j < featureList.size(); j++) {
						if (i == j) {
							pos = featureList.get(j);
						} else {
							neg = neg + "!" + featureList.get(j) + " & ";
						}
					}
					neg = neg + lookUpMap.get(parentFeature);
					neg = "( "+neg+" )";
					str = str + "( !"+pos+" | "+neg+" ) & ( !"+neg+" | "+pos+" )";
					str = str + ")";
					if (i < featureList.size() - 1) {
						str = str + " & ";
					}
				}
				str = str + " )";			
			}
		}
		return str;
	}

	@Override
	public void addExcludes(GenericRelation rel, GenericFeature origin,
			GenericFeature destination) {
		if (isTurn()) {
			String p = featuresMap.get(origin.getName());
			String q = featuresMap.get(destination.getName());
			String str = "!( " + p + " & " + q + " )";
			featureConstraintsList.add("( " + str + " )");
		} else {
			String p = componentsMap.get(origin.getName());
			String q = componentsMap.get(destination.getName());
			String str = "!( " + p + " & " + q + " )";
			componentConstraintsList.add("( " + str + " )");
		}
	}

	@Override
	public void addRequires(GenericRelation rel, GenericFeature origin,
			GenericFeature destination) {
		if (isTurn()) {
			String p = featuresMap.get(origin.getName());
			String q = featuresMap.get(destination.getName());
			String str = "!" + p + " | " + q;
			featureConstraintsList.add("( " + str + " )");
		} else {
			String p = componentsMap.get(origin.getName());
			String q = componentsMap.get(destination.getName());
			String str = "!" + p + " | " + q;
			componentConstraintsList.add("( " + str + " )");
		}
	}

	@Override
	public PerformanceResult ask(Question q) {
		QSATQuestion question = (QSATQuestion) q;
		try {
			question.generateQBFFormula(this);
			
			convertQBFToQPRO(question.getQuestionId(),question.prepath);
		} catch (QBFFormulaGenerationException e) {
			e.printStackTrace();
		}
		PerformanceResult performanceResult = runQSATonQBF(question);
		return performanceResult;
	}

	@Override
	public void unapplyStagedConfigurations() {
		// TODO Auto-generated method stub

	}

	@Override
	public void applyStagedConfiguration(Configuration conf) {
		// TODO Auto-generated method stub

	}

	@Override
	public Map<String, Object> getHeusistics() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void setHeuristic(Object obj) {
		// TODO Auto-generated method stub

	}
	
	/*
	public void convertQBFToQPRO(String questionName)
			throws QBFFormulaGenerationException {
		
		try {
			FileWriter fstream = new FileWriter(questionName+".qpro");
			Process p = Runtime.getRuntime().exec("./boole2qpro < "+questionName + ".qbf");
			BufferedReader br = new BufferedReader(new InputStreamReader(p.getErrorStream()));
			BufferedWriter out = new BufferedWriter(fstream);
			String line = null;
			while ((line = br.readLine()) != null) {
				System.out.println(line);
				out.write(line);
			}
			out.close();
			br.close();
			p.waitFor();
		} catch (Exception e) {
			throw new QBFFormulaGenerationException(e.getMessage(),
					QSATReasoner.class.toString());
		}

	}
	*/
	
	
	
	public void convertQBFToQPRO(String questionName, String prepath)
			throws QBFFormulaGenerationException {
		try {
			String exec="./boole2qpro < "+prepath+questionName + ".qbf > " +prepath+ questionName + ".qpro";
			Random r = new Random();
			int nextInt = r.nextInt(1000000);
			File f = new File("./"+nextInt+QBFFormulaConstants.SHELL_FILE_BOOLETOQPRO + ".sh");

			FileWriter fstream = new FileWriter(f);
			System.out.println(exec);
				BufferedWriter out = new BufferedWriter(fstream);
			out.append("./boole2qpro < ");
			out.append(prepath+questionName + ".qbf > " + prepath+questionName + ".qpro");
			out.flush();
			out.close();
			
			f.setExecutable(true);

			Process p = Runtime.getRuntime().exec("./"+nextInt+QBFFormulaConstants.SHELL_FILE_BOOLETOQPRO + ".sh");
		
//			Process p = Runtime.getRuntime().exec(exec);
			p.waitFor();
			f.delete();
			
		} catch (Exception e) {
			throw new QBFFormulaGenerationException(e.getMessage(),
					QSATReasoner.class.toString());
		}
	}
	

	private QSATResult runQSATonQBF(Question questionName) {
		QSATQuestion q = (QSATQuestion) questionName;
		QSATResult qSATResult = new QSATResult();
		qSATResult.setResultFlag(true);
		try {
			Process p = Runtime.getRuntime().exec(
					"./" + QBFFormulaConstants.QSAT_SOLVER_NAME + " "
							+ q.prepath+ q.getQuestionId() + ".qpro");
			BufferedReader br = new BufferedReader(new InputStreamReader(
					p.getErrorStream()));
			String line = null;
			String output = "";
			while ((line = br.readLine()) != null) {
				if(line!=null && line.contains("no solution found"))
				{
					qSATResult.setResultFlag(false);
				}
				output += line + "\n";
			}
			p.waitFor();
			//System.out.println("flag:"+qSATResult.isResultFlag());
			qSATResult.setQsatDump(output);
		} catch (Exception e) {
			// throw new QBFFormulaGenerationException(e.getMessage(),
			// QSATReasoner.class.toString());
		}
		return qSATResult;
	}

	@Override
	public void addTraceability(Map<String, List<List<String>>> trace) {
		Iterator<Map.Entry<String, List<List<String>>>> entries = trace
				.entrySet().iterator();
		while (entries.hasNext()) {
			String traceFormula = new String("( ");
			Map.Entry<String, List<List<String>>> entry = entries.next();
			String fVal = featuresMap.get(entry.getKey());
			List<List<String>> allCompList = entry.getValue();
			Iterator<List<String>> allCompListItr = allCompList.iterator();
			while (allCompListItr.hasNext()) {
				List<String> compList = allCompListItr.next();
				Iterator<String> compListItr = compList.iterator();
				traceFormula = traceFormula + "( ";
				while (compListItr.hasNext()) {
					String component = compListItr.next();
					String cVal = componentsMap.get(component);
					traceFormula = traceFormula + cVal + " ";
					if (compListItr.hasNext()) {
						traceFormula = traceFormula + "& ";
					}
				}
				traceFormula = traceFormula + ") ";
				if (allCompListItr.hasNext()) {
					traceFormula = traceFormula + "| ";
				}
				if(!allCompListItr.hasNext()){
					traceFormula = traceFormula + ")";
				}
			}
			traceability.put(fVal, traceFormula);
		}
	}

	public String generateTraceabilityFormula() {

		Iterator<Map.Entry<String, String>> entries = traceability.entrySet()
				.iterator();
		traceabilityFormula = traceabilityFormula + "( ";
		while (entries.hasNext()) {
			Map.Entry<String, String> entry = entries.next();
			String str = "( !"+entry.getKey()+" | "+entry.getValue()+")";
			traceabilityFormula = traceabilityFormula + str;
			if (entries.hasNext()) {
				traceabilityFormula = traceabilityFormula + " & ";
			}
		}
		traceabilityFormula = traceabilityFormula + ")";
		return traceabilityFormula;
	}

	public String getTraceabilityFormula() {
		return traceabilityFormula;
	}

	public String generateFeatureConstraints() {

		Iterator<String> featureConstIterator = featureConstraintsList
				.iterator();
		featureConstraints = featureConstraints + "( ";
		while (featureConstIterator.hasNext()) {
			String constraints = featureConstIterator.next();
			featureConstraints = featureConstraints + constraints;
			if (featureConstIterator.hasNext()) {
				featureConstraints = featureConstraints + " & ";
			}
		}
		featureConstraints = featureConstraints + ")";
		return featureConstraints;
	}

	public String generateComponentConstraints() {
		Iterator<String> componentConstIterator = componentConstraintsList
				.iterator();
		componentConstraints = componentConstraints + "( ";
		while (componentConstIterator.hasNext()) {
			String constraints = componentConstIterator.next();
			componentConstraints = componentConstraints + constraints;
			if (componentConstIterator.hasNext()) {
				componentConstraints = componentConstraints + " & ";
			}
		}
		componentConstraints = componentConstraints + ")";
		return componentConstraints;
	}
}
