/*
	This file is part of FaMaTS.

    FaMaTS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FaMaTS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with FaMaTS.  If not, see <http://www.gnu.org/licenses/>.

 */
package in.barc.iitb.cfdvs.QSATReasoner;

import java.util.HashMap;
import java.util.Map;

import es.us.isa.FAMA.Benchmarking.PerformanceResult;

/**
 * This file captures the results form QSAT Solver-CirQit.
 * 
 * @author Ganesh N
 */
public class QSATResult extends PerformanceResult {

	private Integer propagations;
	private Integer learnedliterals;
	private Integer changedreason;
	private Integer reduceddb;
	private Integer starts;
	private Integer learnedbinaryclauses;
	private Integer learnedclauses;
	private Integer reducedliterals;
	private Integer learnedternaryclauses;
	private Integer conflicts;
	private Integer inspects;
	private Integer rootsimplifications;
	private Integer decisions;
	private Integer variables;
	private Integer clauses;
	private String qsatDump;
	private boolean resultFlag;

	private final static String header = "CirQit:";

	/**
	 * Default constructor which initialise default values for all the result
	 * parameter.
	 */
	public QSATResult() {
		propagations = new Integer(404);
		learnedliterals = new Integer(404);
		changedreason = new Integer(404);
		reduceddb = new Integer(404);
		starts = new Integer(404);
		learnedbinaryclauses = new Integer(404);
		learnedclauses = new Integer(404);
		reducedliterals = new Integer(404);
		learnedternaryclauses = new Integer(404);
		conflicts = new Integer(404);
		inspects = new Integer(404);
		rootsimplifications = new Integer(404);
		decisions = new Integer(404);
		resultFlag = true;
	}

	/**
	 * 
	 */
	public String toString() {
		String res = "Main QSAT Statistics:\n" + "Time (ms): " + this.getTime()
				+ "\n";
		return res;
	}

	/**
	 * 
	 */
	@Override
	public Map<String, String> getResults() {
		Map<String, String> res = new HashMap<String, String>();
		res.put(header + "time", String.valueOf(time));
		res.put(header + "propagations", String.valueOf(propagations));
		res.put(header + "learnedliterals", String.valueOf(learnedliterals));
		res.put(header + "changedreason", String.valueOf(changedreason));
		res.put(header + "reduceddb", String.valueOf(reduceddb));
		res.put(header + "starts", String.valueOf(starts));
		res.put(header + "learnedbinaryclauses",
				String.valueOf(learnedbinaryclauses));
		res.put(header + "learnedclauses", String.valueOf(learnedclauses));
		res.put(header + "reducedliterals", String.valueOf(reducedliterals));
		res.put(header + "learnedternaryclauses",
				String.valueOf(learnedternaryclauses));
		res.put(header + "conflicts", String.valueOf(conflicts));
		res.put(header + "inspects", String.valueOf(inspects));
		res.put(header + "rootsimplifications",
				String.valueOf(rootsimplifications));
		res.put(header + "decisions", String.valueOf(decisions));
		res.put(header + "variables", "" + variables);
		res.put(header + "clauses", "" + clauses);
		res.put(header + "qsatDump", "" + qsatDump);
		return res;
	}

	public Integer getPropagations() {
		return propagations;
	}

	public void setPropagations(Integer propagations) {
		this.propagations = propagations;
	}

	public Integer getLearnedliterals() {
		return learnedliterals;
	}

	public void setLearnedliterals(Integer learnedliterals) {
		this.learnedliterals = learnedliterals;
	}

	public Integer getChangedreason() {
		return changedreason;
	}

	public void setChangedreason(Integer changedreason) {
		this.changedreason = changedreason;
	}

	public Integer getReduceddb() {
		return reduceddb;
	}

	public void setReduceddb(Integer reduceddb) {
		this.reduceddb = reduceddb;
	}

	public Integer getStarts() {
		return starts;
	}

	public void setStarts(Integer starts) {
		this.starts = starts;
	}

	public Integer getLearnedbinaryclauses() {
		return learnedbinaryclauses;
	}

	public void setLearnedbinaryclauses(Integer learnedbinaryclauses) {
		this.learnedbinaryclauses = learnedbinaryclauses;
	}

	public Integer getLearnedclauses() {
		return learnedclauses;
	}

	public void setLearnedclauses(Integer learnedclauses) {
		this.learnedclauses = learnedclauses;
	}

	public Integer getReducedliterals() {
		return reducedliterals;
	}

	public void setReducedliterals(Integer reducedliterals) {
		this.reducedliterals = reducedliterals;
	}

	public Integer getLearnedternaryclauses() {
		return learnedternaryclauses;
	}

	public void setLearnedternaryclauses(Integer learnedternaryclauses) {
		this.learnedternaryclauses = learnedternaryclauses;
	}

	public Integer getConflicts() {
		return conflicts;
	}

	public void setConflicts(Integer conflicts) {
		this.conflicts = conflicts;
	}

	public Integer getInspects() {
		return inspects;
	}

	public void setInspects(Integer inspects) {
		this.inspects = inspects;
	}

	public Integer getRootsimplifications() {
		return rootsimplifications;
	}

	public void setRootsimplifications(Integer rootsimplifications) {
		this.rootsimplifications = rootsimplifications;
	}

	public Integer getDecisions() {
		return decisions;
	}

	public void setDecisions(Integer decisions) {
		this.decisions = decisions;
	}

	public Integer getVariables() {
		return variables;
	}

	public void setVariables(Integer variables) {
		this.variables = variables;
	}

	public Integer getClauses() {
		return clauses;
	}

	public void setClauses(Integer clauses) {
		this.clauses = clauses;
	}

	@Override
	public void addFields(PerformanceResult res) {
		// TODO Auto-generated method stub

	}

	public String getQsatDump() {
		return qsatDump;
	}

	public void setQsatDump(String qsatDump) {
		this.qsatDump = qsatDump;
	}

	public boolean isResultFlag() {
		return resultFlag;
	}

	public void setResultFlag(boolean resultFlag) {
		this.resultFlag = resultFlag;
	}

}
