package in.barc.iitb.cfdvs.QSATReasoner.questions;

import in.barc.iitb.cfdvs.QSATConstants.QBFFormulaConstants;
import in.barc.iitb.cfdvs.QSATException.QBFFormulaGenerationException;
import in.barc.iitb.cfdvs.QSATReasoner.QSATQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.QSATReasoner;
import in.barc.iitb.cfdvs.SPLANE.questionInterfaces.IIntersectionQuestion;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class IntersectionQuestion extends QSATQuestion implements IIntersectionQuestion {

	private List<String> inputOne;
	private List<String> inputTwo;
	Map<String, List<String>> inputMapOne;
	Map<String, List<String>> inputMapTwo;
	private String inputType;

	public IntersectionQuestion(List<String> inputOne, List<String> inputTwo,
			String inputType) {
		questionId = QBFFormulaConstants.FORMULA_NAME_INTERSECTION;
		this.inputOne = inputOne;
		this.inputTwo = inputTwo;
		this.inputType = inputType;
	}
	
	public IntersectionQuestion(Map<String, List<String>> inputMapOne, Map<String, List<String>> inputMapTwo,
			String inputType) {
		questionId = QBFFormulaConstants.FORMULA_NAME_INTERSECTION;
		this.inputMapOne = inputMapOne;
		this.inputMapTwo = inputMapTwo;
		this.inputType = inputType;
	}
	
	

	public void generateQBFFormula(QSATReasoner reasoner)
			throws QBFFormulaGenerationException {
		try {
			FileWriter fstream = new FileWriter(prepath+
					QBFFormulaConstants.FORMULA_NAME_INTERSECTION + ".qbf");
			BufferedWriter out = new BufferedWriter(fstream);
			if (QBFFormulaConstants.INPUT_TYPE_SPECIFICATION
					.equalsIgnoreCase(inputType)) {
				intersectionSpecifications(reasoner, out);
			} else if (QBFFormulaConstants.INPUT_TYPE_IMPLEMENTATION
					.equalsIgnoreCase(inputType)) {
				intersectionImplementations(reasoner, out);
			} else if (QBFFormulaConstants.INPUT_TYPE_PRODUCT
					.equalsIgnoreCase(inputType)) {
				intersectionProducts(reasoner, out);
			} else {
				new QBFFormulaGenerationException(
						"Invalid input type:"
								+ inputType
								+ "\nValid types are:\n S: specification \n I: Implementation \n P: product",
						IntersectionQuestion.class.toString());
			}
			out.close();
		} catch (Exception e) {
			throw new QBFFormulaGenerationException(e.getMessage(),
					IntersectionQuestion.class.toString());
		}
	}

	private void intersectionSpecifications(QSATReasoner reasoner, BufferedWriter out)
			throws QBFFormulaGenerationException {
		try {
			validFeatures(inputOne, reasoner.getFeaturesMap());
			validFeatures(inputTwo, reasoner.getFeaturesMap());
			out.append(QBFFormulaConstants.QUANTIFIER_EXISTS + "[ ");
			Iterator<Map.Entry<String, String>> entries = reasoner
					.getFeaturesMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append("p" + entry.getValue() + " ");
			}
			entries = reasoner.getFeaturesMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append("d" + entry.getValue() + " ");
			}
			entries = reasoner.getFeaturesMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append(entry.getValue() + " ");
			}
			out.append("] (");

			entries = reasoner.getFeaturesMap().entrySet().iterator();
			out.append("( ");
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				String feature = entry.getKey();
				if (inputOne.contains(feature))
					out.append("p" + entry.getValue());
				else
					out.append("!p" + entry.getValue());
				if (entries.hasNext()) {
					out.append(" & ");
				}
			}
			out.append(" )");
			out.append(" & ");

			entries = reasoner.getFeaturesMap().entrySet().iterator();
			out.append("( ");
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				String feature = entry.getKey();
				if (inputTwo.contains(feature))
					out.append("d" + entry.getValue());
				else
					out.append("!d" + entry.getValue());
				if (entries.hasNext()) {
					out.append(" & ");
				}
			}
			out.append(" )");
			out.append(" & ");

			out.append("(");
			entries = reasoner.getFeaturesMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append("(");
				out.append("( !" + entry.getValue() + " | ( p"
						+ entry.getValue() + " | d" + entry.getValue() + ") )");
				out.append(" & ");
				out.append("( " + entry.getValue() + " | !( p"
						+ entry.getValue() + " | d" + entry.getValue() + ") )");
				out.append(")");
				if (entries.hasNext())
					out.append(" & ");
			}
			out.append(") & " + reasoner.getFeatureConstraints());
			out.append(" )");
		} catch (Exception e) {
			throw new QBFFormulaGenerationException(
					"Error while intersection specificaiton : " + e.getMessage(),
					IntersectionQuestion.class.toString());
		}
	}

	private void intersectionImplementations(QSATReasoner reasoner, BufferedWriter out)
			throws QBFFormulaGenerationException {
		try {
			validComponents(inputOne, reasoner.getComponentsMap());
			validComponents(inputTwo, reasoner.getComponentsMap());
			out.append(QBFFormulaConstants.QUANTIFIER_EXISTS + "[ ");
			Iterator<Map.Entry<String, String>> entries = reasoner
					.getComponentsMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append("p" + entry.getValue() + " ");
			}
			entries = reasoner.getComponentsMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append("d" + entry.getValue() + " ");
			}
			entries = reasoner.getComponentsMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append(entry.getValue() + " ");
			}
			out.append("] (");

			entries = reasoner.getComponentsMap().entrySet().iterator();
			out.append("( ");
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				String component = entry.getKey();
				if (inputOne.contains(component))
					out.append("p" + entry.getValue());
				else
					out.append("!p" + entry.getValue());
				if (entries.hasNext()) {
					out.append(" & ");
				}
			}
			out.append(" )");
			out.append(" & ");

			entries = reasoner.getComponentsMap().entrySet().iterator();
			out.append("( ");
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				String component = entry.getKey();
				if (inputTwo.contains(component))
					out.append("d" + entry.getValue());
				else
					out.append("!d" + entry.getValue());
				if (entries.hasNext()) {
					out.append(" & ");
				}
			}
			out.append(" )");
			out.append(" & ");

			out.append("(");
			entries = reasoner.getComponentsMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append("(");
				out.append("( !" + entry.getValue() + " | ( p"
						+ entry.getValue() + " | d" + entry.getValue() + ") )");
				out.append(" & ");
				out.append("( " + entry.getValue() + " | !( p"
						+ entry.getValue() + " | d" + entry.getValue() + ") )");
				out.append(")");
				if (entries.hasNext())
					out.append(" & ");
			}
			out.append(") & " + reasoner.getComponentConstraints());
			out.append(" )");
		} catch (Exception e) {
			throw new QBFFormulaGenerationException(
					"Error while intersection implementation : " + e.getMessage(),
					IntersectionQuestion.class.toString());
		}
	}
	
	
	private void intersectionProducts(QSATReasoner reasoner, BufferedWriter out)
			throws QBFFormulaGenerationException {
		try {
			
			List<String> spec1 = inputMapOne.get(QBFFormulaConstants.PRODUCT_SPECIFICATION);
			List<String> spec2 = inputMapTwo.get(QBFFormulaConstants.PRODUCT_SPECIFICATION);
			List<String> impl1 = inputMapOne.get(QBFFormulaConstants.PRODUCT_IMPLEMENTATION);
			List<String> impl2 = inputMapTwo.get(QBFFormulaConstants.PRODUCT_IMPLEMENTATION);
			
			validFeatures(spec1, reasoner.getFeaturesMap());
			validFeatures(spec2, reasoner.getFeaturesMap());			
			validComponents(impl1, reasoner.getComponentsMap());
			validComponents(impl2, reasoner.getComponentsMap());
			
			out.append(QBFFormulaConstants.QUANTIFIER_EXISTS + "[ ");
			Iterator<Map.Entry<String, String>> entries = reasoner
					.getFeaturesMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append("p1" + entry.getValue() + " ");
			}
			entries = reasoner
					.getComponentsMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append("d1" + entry.getValue() + " ");
			}
			entries = reasoner
					.getFeaturesMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append("p2" + entry.getValue() + " ");
			}			
			entries = reasoner.getComponentsMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append("d2" + entry.getValue() + " ");
			}
			entries = reasoner
					.getFeaturesMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append(entry.getValue() + " ");
			}
			entries = reasoner.getComponentsMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append(entry.getValue() + " ");
			}
			out.append("] (");

			
			entries = reasoner.getFeaturesMap().entrySet().iterator();
			out.append("( ");
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				String feature = entry.getKey();
				if (spec1.contains(feature))
					out.append("p1" + entry.getValue());
				else
					out.append("!p1" + entry.getValue());
				if (entries.hasNext()) {
					out.append(" & ");
				}
			}
			out.append(" & ");
			entries = reasoner.getComponentsMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				String component = entry.getKey();
				if (impl1.contains(component))
					out.append("d1" + entry.getValue());
				else
					out.append("!d1" + entry.getValue());
				if (entries.hasNext()) {
					out.append(" & ");
				}
			}
			out.append(" )");
			out.append(" & ");

			entries = reasoner.getFeaturesMap().entrySet().iterator();
			out.append("( ");
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				String feature = entry.getKey();
				if (spec2.contains(feature))
					out.append("p2" + entry.getValue());
				else
					out.append("!p2" + entry.getValue());
				if (entries.hasNext()) {
					out.append(" & ");
				}
			}
			out.append(" & ");
			entries = reasoner.getComponentsMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				String component = entry.getKey();
				if (impl2.contains(component))
					out.append("d2" + entry.getValue());
				else
					out.append("!d2" + entry.getValue());
				if (entries.hasNext()) {
					out.append(" & ");
				}
			}
			out.append(" )");
			out.append(" & ");

			out.append("(");
			entries = reasoner.getFeaturesMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append("(");
				out.append("( !" + entry.getValue() + " | ( p1"
						+ entry.getValue() + " | p2" + entry.getValue() + ") )");
				out.append(" & ");
				out.append("( " + entry.getValue() + " | !( p1"
						+ entry.getValue() + " | p2" + entry.getValue() + ") )");
				out.append(")");
				if (entries.hasNext())
					out.append(" & ");
			}
			out.append(" & ");
			entries = reasoner.getComponentsMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append("(");
				out.append("( !" + entry.getValue() + " | ( d1"
						+ entry.getValue() + " | d2" + entry.getValue() + ") )");
				out.append(" & ");
				out.append("( " + entry.getValue() + " | !( d1"
						+ entry.getValue() + " | d2" + entry.getValue() + ") )");
				out.append(")");
				if (entries.hasNext())
					out.append(" & ");
			}
			out.append(") & " + reasoner.getComponentConstraints() + " & "+reasoner.getFeatureConstraints());
			out.append(" )");
		} catch (Exception e) {
			throw new QBFFormulaGenerationException(
					"Error while intersection products : " + e.getMessage(),
					IntersectionQuestion.class.toString());
		}
	}
	
}
