package in.barc.iitb.cfdvs.QSATReasoner.questions;

import in.barc.iitb.cfdvs.QSATConstants.QBFFormulaConstants;
import in.barc.iitb.cfdvs.QSATException.QBFFormulaGenerationException;
import in.barc.iitb.cfdvs.QSATReasoner.QSATQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.QSATReasoner;
import in.barc.iitb.cfdvs.SPLANE.questionInterfaces.IRealizesQuestion;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class RealizesQuestion extends QSATQuestion implements IRealizesQuestion {

	private List<String> implementation;
	private List<String> specification;

	public RealizesQuestion(List<String> implConf, List<String> specConf) {
		questionId = QBFFormulaConstants.FORMULA_NAME_REALIZES;
		implementation = implConf;
		specification = specConf;
	}

	public void generateQBFFormula(QSATReasoner reasoner)
			throws QBFFormulaGenerationException {
		try {
			validComponents(implementation, reasoner.getComponentsMap());
			validFeatures(specification, reasoner.getFeaturesMap());
			FileWriter fstream = new FileWriter(prepath+
					QBFFormulaConstants.FORMULA_NAME_REALIZES + ".qbf");
			BufferedWriter out = new BufferedWriter(fstream);
			out.append(QBFFormulaConstants.QUANTIFIER_EXISTS + "[ ");
			Iterator<Map.Entry<String, String>> entries = reasoner
					.getComponentsMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append(entry.getValue() + " ");
			}
			entries = reasoner.getFeaturesMap().entrySet().iterator();
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append(entry.getValue() + " ");
			}
			out.append("] (");
			entries = reasoner.getComponentsMap().entrySet().iterator();
			out.append("( ");
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				String component = entry.getKey();
				if (implementation.contains(component))
					out.append(entry.getValue());
				else
					out.append("!" + entry.getValue());
				if (entries.hasNext()) {
					out.append(" & ");
				}
			}
			out.append(" ) & ");

			entries = reasoner.getFeaturesMap().entrySet().iterator();
			out.append("( ");
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				String feature = entry.getKey();
				if (specification.contains(feature))
					out.append(entry.getValue());
				else
					out.append("!" + entry.getValue());
				if (entries.hasNext()) {
					out.append(" & ");
				}
			}
			out.append(" ) &");
			entries = reasoner.getFeaturesMap().entrySet().iterator();
			out.append("( ");
			while (entries.hasNext()) {
				Map.Entry<String, String> entry = entries.next();
				out.append("( ( !" + entry.getValue() + " | ("
						+ reasoner.getTraceability().get(entry.getValue())
						+ ") ) & ( " + entry.getValue() + " | !("
						+ reasoner.getTraceability().get(entry.getValue())
						+ ") ))");
				if (entries.hasNext()) {
					out.append(" & ");
				}
			}
			out.append(" )");
			out.append(" )");
			out.close();
		} catch (Exception e) {
			throw new QBFFormulaGenerationException(e.getMessage(),
					CoversQuestion.class.toString());
		}
	}

}