package in.barc.iitb.cfdvs.SPLANE.Question.Junit;

import in.barc.iitb.cfdvs.QSATReasoner.QSATReasoner;
import in.barc.iitb.cfdvs.QSATReasoner.QSATResult;
import in.barc.iitb.cfdvs.QSATReasoner.questions.CommonQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.CompletenessQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.DeadQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.LiveQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.SoundnessQuestion;
import in.barc.iitb.cfdvs.SPLANE.models.parser.TraceabilityParser;
import in.barc.iitb.cfdvs.SPLANEModel.transformations.SplaneModelTransform;
import junit.framework.TestCase;

import org.junit.Test;

import es.us.isa.FAMA.models.FAMAfeatureModel.fileformats.FMPlainTextReader;
import es.us.isa.FAMA.models.variabilityModel.TraceabilityModel;
import es.us.isa.FAMA.models.variabilityModel.VariabilityModel;

public class JunitTest extends TestCase {

	protected QSATReasoner reasoner;

	/*
	 * (non-Javadoc)
	 * 
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() {
		//System.out.println("In setup");
		String featureFileName = "//home//gnarwane//Desktop//splaneinfama//splane-fama//trunk//QSATReasoner//HIS.fm";
		String componentFileName = "//home//gnarwane//Desktop//splaneinfama//splane-fama//trunk//QSATReasoner//HISc.fm";
		String traceFileName = "//home//gnarwane//Desktop//splaneinfama//splane-fama//trunk//QSATReasoner//HISt.fm";

		FMPlainTextReader reader = new FMPlainTextReader();
		VariabilityModel fm = null;
		try {
			fm = reader.parseFile(featureFileName);
		} catch (Exception e) {
			e.printStackTrace();
		}

		reader = new FMPlainTextReader();
		VariabilityModel cm = null;
		try {
			cm = reader.parseFile(componentFileName);
		} catch (Exception e) {
			e.printStackTrace();
		}

		SplaneModelTransform t = new SplaneModelTransform();
		TraceabilityParser trace = new TraceabilityParser(traceFileName);
		trace.parse();
		TraceabilityModel traceModel = new TraceabilityModel();
		traceModel.setTraceabilityOrgMap(trace.getTraProvidesMap());

		reasoner = new QSATReasoner();
		t.transform(fm, cm, traceModel, reasoner);
	}

	@Test
	public void testCommonQuestionValid() {

		System.out
				.println("---------------testCommonQuestionValid---------------");
		CommonQuestion q = new CommonQuestion("CONTROL");
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		System.out.println(qsatResult.getQsatDump());
	}

	@Test
	public void testCommonQuestionInvalid() {

		System.out
				.println("---------------testCommonQuestionInvalid---------------");
		CommonQuestion q = new CommonQuestion("SERVICES");
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		System.out.println(qsatResult.getQsatDump());
	}
	
	@Test
	public void testCompletenessQuestion() {

		System.out
				.println("---------------testCompletenessQuestion---------------");
		CompletenessQuestion q = new CompletenessQuestion();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		System.out.println(qsatResult.getQsatDump());
	}
	
	@Test
	public void testSoundnessQuestion() {

		System.out
				.println("---------------testSoundnessQuestion---------------");
		SoundnessQuestion q = new SoundnessQuestion();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		System.out.println(qsatResult.getQsatDump());
	}
	
	@Test
	public void testLiveQuestion() {

		System.out
				.println("---------------testLiveQuestion---------------");
		LiveQuestion q = new LiveQuestion("CONTROL");
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		System.out.println(qsatResult.getQsatDump());
	}

	@Test
	public void testDeadQuestion() {

		System.out
				.println("---------------testDeadQuestion---------------");
		DeadQuestion q = new DeadQuestion("CONTROL");
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		System.out.println(qsatResult.getQsatDump());
	}
}
