package in.barc.iitb.cfdvs.SPLANE.Question.Junit;

import in.barc.iitb.cfdvs.NonSolverReasoner.NonSolverReasoner;
import in.barc.iitb.cfdvs.NonSolverReasoner.NonSolverResult;
import in.barc.iitb.cfdvs.NonSolverReasoner.questions.CompleteTraceabilityQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.QSATReasoner;
import in.barc.iitb.cfdvs.QSATReasoner.QSATResult;
import in.barc.iitb.cfdvs.QSATReasoner.questions.CommonQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.CompletenessQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.CoversQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.CriticalQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.DeadQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.ImplementsQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.LiveQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.RealizesQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.SoundnessQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.UnionQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.ValidComponentModelQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.ValidFeatureModelQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.ValidImplementationQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.ValidModelQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.ValidSpecificationQuestion;
import in.barc.iitb.cfdvs.QSATReasoner.questions.VoidProductModelQuestion;
import in.barc.iitb.cfdvs.SPLANE.models.parser.TraceabilityParser;
import in.barc.iitb.cfdvs.SPLANEModel.transformations.SplaneModelTransform;

import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;
import es.us.isa.FAMA.models.FAMAfeatureModel.fileformats.FMPlainTextReader;
import es.us.isa.FAMA.models.variabilityModel.TraceabilityModel;
import es.us.isa.FAMA.models.variabilityModel.VariabilityModel;

public class MobilePhoneJunit extends TestCase {

	protected QSATReasoner reasoner;

	/*
	 * (non-Javadoc)
	 * 
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() {
		// System.out.println("In setup");
		String featureFileName = "fMobilePhone.afm";
		String componentFileName = "cMobilePhone.afm";
		String traceFileName = "tMobilePhone.afm";

		FMPlainTextReader reader = new FMPlainTextReader();
		VariabilityModel fm = null;
		try {
			fm = reader.parseFile(featureFileName);
		} catch (Exception e) {
			e.printStackTrace();
		}

		reader = new FMPlainTextReader();
		VariabilityModel cm = null;
		try {
			cm = reader.parseFile(componentFileName);
		} catch (Exception e) {
			e.printStackTrace();
		}

		SplaneModelTransform t = new SplaneModelTransform();
		TraceabilityParser trace = new TraceabilityParser(traceFileName);
		trace.parse();
		TraceabilityModel traceModel = new TraceabilityModel();
		traceModel.setTraceabilityOrgMap(trace.getTraProvidesMap());

		reasoner = new QSATReasoner();
		t.transform(fm, cm, traceModel, reasoner);
		// System.out.println("Feature Constraints:"+reasoner.getFeatureConstraints());
		// System.out.println("Component Constraints:"+reasoner.getComponentConstraints());
		// System.out.println("Feature Map:"+reasoner.getFeaturesMap());
		// System.out.println("Component Map:"+reasoner.getComponentsMap());
		// System.out.println("Traceability Map:"+reasoner.getTraceability());
		// System.out.println("Traceability Formula :"+reasoner.getTraceabilityFormula());

	}

	// Tested and working.
	public void testValidFeatureModelQuestion() {

		System.out
				.println("---------------testValidFeatureModelQuestion---------------");
		ValidFeatureModelQuestion q = new ValidFeatureModelQuestion();
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println("Number of Features:"+reasoner.getFeaturesMap().size());
		System.out.println("Number of Components:"+reasoner.getComponentsMap().size());
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testValidComponentModelQuestion() {

		System.out
				.println("---------------testValidComponentModelQuestion---------------");
		ValidComponentModelQuestion q = new ValidComponentModelQuestion();
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testValidModelQuestion() {

		System.out
				.println("---------------testValidModelQuestion---------------");
		ValidModelQuestion q = new ValidModelQuestion();
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testVoidProductModelQuestion() {

		System.out
				.println("---------------testVoidProductModelQuestion---------------");
		VoidProductModelQuestion q = new VoidProductModelQuestion();
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testCompleteTraceabilityQuestion() {

		System.out
				.println("---------------testCompleteTraceabilityQuestion---------------");
		CompleteTraceabilityQuestion q = new CompleteTraceabilityQuestion();
		NonSolverReasoner nonSolverReasoner = new NonSolverReasoner();
		long startTime = System.currentTimeMillis();
		NonSolverResult nonSolverResult = nonSolverReasoner
				.execute(q, reasoner);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(nonSolverResult.getResultDump());
	}

	// Tested and working
	public void testImplementationQuestionValid() {

		System.out
				.println("---------------testImplementationQuestionValid---------------");

		List<String> implementation = getMinimumComponents();
		implementation.add("SMSApps");
		implementation.add("SymbianOSSetup");
		implementation.add("MIDP20");
		String featureName = "SMS";
		ImplementsQuestion q = new ImplementsQuestion(implementation,
				featureName);
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testImplementationQuestionInvalid() {
		System.out
				.println("---------------testImplementationQuestionInvalid---------------");
		List<String> implementation = getMinimumComponents();
		implementation.add("SMSApps");
		implementation.add("SymbianOSSetup");
		implementation.add("MIDP20");
		String featureName = "EMS";
		ImplementsQuestion q = new ImplementsQuestion(implementation,
				featureName);
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testCommonQuestionValid() {

		System.out
				.println("---------------testCommonQuestionValid---------------");
		CommonQuestion q = new CommonQuestion("Calls");
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testCommonQuestionInvalid() {

		System.out
				.println("---------------testCommonQuestionInvalid---------------");
		CommonQuestion q = new CommonQuestion("MP3");
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testLiveQuestion() {

		System.out.println("---------------testLiveQuestion---------------");
		LiveQuestion q = new LiveQuestion("Games");
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testDeadQuestion() {

		System.out.println("---------------testDeadQuestion---------------");
		DeadQuestion q = new DeadQuestion("Games");
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	// TODO test with small example
	public void testCompletenessQuestion() {

		System.out
				.println("---------------testCompletenessQuestion---------------");
		CompletenessQuestion q = new CompletenessQuestion();
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	// TODO test with small example
	public void testSoundnessQuestion() {

		System.out
				.println("---------------testSoundnessQuestion---------------");
		SoundnessQuestion q = new SoundnessQuestion();
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testValidImplementationQuestionValid() {

		System.out
				.println("---------------testValidImplementationQuestionValid---------------");

		List<String> implementation = getMinimumComponents();
		implementation.add("SMSApps");
		implementation.add("EMSApps");
		implementation.add("SymbianOSSetup");
		implementation.add("MIDP20");
		ValidImplementationQuestion q = new ValidImplementationQuestion(
				implementation);
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testValidImplementationQuestionInvalid() {

		System.out
				.println("---------------testValidImplementationQuestionInvalid---------------");

		List<String> implementation = getMinimumComponents();
		implementation.add("SMSApps");
		implementation.add("SymbianOSSetup");
		ValidImplementationQuestion q = new ValidImplementationQuestion(
				implementation);
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testValidSpecificationQuestionValid() {

		System.out.println("----testValidSpecificationQuestionValid----");

		List<String> specification = getMinimumFeatures();
		specification.add("Data");
		specification.add("Symbian");
		specification.add("SMS");
		ValidSpecificationQuestion q = new ValidSpecificationQuestion(
				specification);
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testValidSpecificationQuestionInvalid() {

		System.out
				.println("---------------testValidSpecificationQuestionInvalid---------------");

		List<String> specification = getMinimumFeatures();
		specification.add("Data");
		specification.add("Symbian");
		ValidSpecificationQuestion q = new ValidSpecificationQuestion(
				specification);
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testCoversQuestionValid() {

		System.out
				.println("---------------testCoversQuestionValid---------------");
		List<String> implementation = getMinimumComponents();
		implementation.add("SMSApps");
		implementation.add("EMSApps");
		implementation.add("MMSApps");
		implementation.add("SymbianOSSetup");
		implementation.add("MIDP20");

		List<String> specification = getMinimumFeatures();
		specification.add("Voice");
		specification.add("Symbian");
		specification.add("SMS");

		CoversQuestion q = new CoversQuestion(implementation, specification);
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testCoversQuestionInvalid() {

		System.out
				.println("---------------testCoversQuestionInvalid---------------");

		// Create Implementation
		List<String> implementation = getMinimumComponents();
		implementation.add("SMSApps");
		implementation.add("SymbianOSSetup");
		implementation.add("MIDP20");

		// Create Specification
		List<String> specification = getMinimumFeatures();
		specification.add("Voice");
		specification.add("Data");
		specification.add("Symbian");
		specification.add("SMS");

		CoversQuestion q = new CoversQuestion(implementation, specification);
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testRealizeQuestionValid() {

		System.out
				.println("---------------testRealizeQuestionValid---------------");

		// Create Implementation
		List<String> implementation = getMinimumComponents();
		implementation.add("SMSApps");
		implementation.add("EMSApps");
		implementation.add("SymbianOSSetup");
		implementation.add("MIDP20");

		// Create Specification
		List<String> specification = getMinimumFeatures();
		specification.add("Voice");
		specification.add("Symbian");
		specification.add("SMS");
		specification.add("EMS");

		RealizesQuestion q = new RealizesQuestion(implementation, specification);
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testRealizeQuestionInvalid() {

		System.out
				.println("---------------testRealizeQuestionInvalid---------------");

		// Create Implementation
		List<String> implementation = getMinimumComponents();
		implementation.add("SMSApps");
		implementation.add("EMSApps");
		implementation.add("MMSApps");

		implementation.add("SymbianOSSetup");
		implementation.add("MIDP20");

		// Create Specification
		List<String> specification = getMinimumFeatures();
		specification.add("Voice");
		specification.add("Data");
		specification.add("Symbian");
		specification.add("SMS");

		RealizesQuestion q = new RealizesQuestion(implementation, specification);
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testCriticalQuestionValid() {

		System.out
				.println("---------------testCriticalQuestionValid---------------");
		String featureName = "SMS";
		String criticalComponent = "SMSApps";
		CriticalQuestion q = new CriticalQuestion(featureName,
				criticalComponent);
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testCriticalQuestionInvalid() {

		System.out
				.println("---------------testCriticalQuestionInvalid---------------");
		String featureName = "SMS";
		String criticalComponent = "EMSApps";
		CriticalQuestion q = new CriticalQuestion(featureName,
				criticalComponent);
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testUnionSpecificationQuestionValid() {

		System.out
				.println("---------------testUnionSpecificationQuestionValid---------------");

		// Create Implementation
		List<String> specificationOne = getMinimumFeatures();
		specificationOne.add("Voice");
		specificationOne.add("Symbian");
		specificationOne.add("EMS");
		specificationOne.add("Media");

		// Create Specification
		List<String> specificationTwo = getMinimumFeatures();
		specificationTwo.add("Voice");
		specificationTwo.add("Data");
		specificationTwo.add("Symbian");
		specificationTwo.add("SMS");

		UnionQuestion q = new UnionQuestion(specificationOne, specificationTwo,
				"S");
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testUnionSpecificationQuestionInvalid() {

		System.out
				.println("---------------testUnionSpecificationQuestionInvalid---------------");

		// Create Implementation
		List<String> specificationOne = getMinimumFeatures();
		specificationOne.add("Voice");
		specificationOne.add("Symbian");
		specificationOne.add("EMS");
		specificationOne.add("Media");
		specificationOne.add("MP4");

		// Create Specification
		List<String> specificationTwo = getMinimumFeatures();
		specificationTwo.add("Voice");
		specificationTwo.add("Data");
		specificationTwo.add("Symbian");
		specificationTwo.add("SMS");
		specificationTwo.add("Media");
		specificationTwo.add("MP3");

		UnionQuestion q = new UnionQuestion(specificationOne, specificationTwo,
				"S");
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testUnionImplementationValid() {

		System.out
				.println("---------------testUnionImplementationValid---------------");
		// Create Implementation
		List<String> implementationOne = getMinimumComponents();
		implementationOne.add("SMSApps");
		implementationOne.add("SymbianOSSetup");
		implementationOne.add("MIDP20");

		// Create Implementation
		List<String> implementationTwo = getMinimumComponents();
		implementationTwo.add("SMSApps");
		implementationTwo.add("SymbianOSSetup");
		implementationTwo.add("MIDP20");
		implementationTwo.add("MediaInterface");
		implementationTwo.add("MediaAdapter");

		UnionQuestion q = new UnionQuestion(implementationOne,
				implementationTwo, "I");
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}

	// Tested and working
	public void testUnionImplementationInvalid() {

		System.out
				.println("---------------testUnionImplementationInvalid---------------");

		// Create Implementation
		List<String> implementationOne = getMinimumComponents();
		implementationOne.add("SMSApps");
		implementationOne.add("SymbianOSSetup");
		implementationOne.add("MIDP20");

		// Create Implementation
		List<String> implementationTwo = getMinimumComponents();
		implementationTwo.add("SMSApps");
		implementationTwo.add("WinCEOSSetup");
		implementationTwo.add("MIDP20");

		UnionQuestion q = new UnionQuestion(implementationOne,
				implementationTwo, "I");
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}
	
	
	
	// Tested and working
	public void testUnionProductValid() {

		System.out
				.println("---------------testUnionProductValid---------------");
		// Create Implementation
		List<String> implementationOne = getMinimumComponents();
		implementationOne.add("SMSApps");
		implementationOne.add("SymbianOSSetup");
		implementationOne.add("MIDP20");

		// Create Implementation
		List<String> implementationTwo = getMinimumComponents();
		implementationTwo.add("SMSApps");
		implementationTwo.add("SymbianOSSetup");
		implementationTwo.add("MIDP20");
		implementationTwo.add("MediaInterface");
		implementationTwo.add("MediaAdapter");

		UnionQuestion q = new UnionQuestion(implementationOne,
				implementationTwo, "I");
		long startTime = System.currentTimeMillis();
		QSATResult qsatResult = (QSATResult) reasoner.ask(q);
		long stopTime = System.currentTimeMillis();
		System.out.println("Execution Time:"+(stopTime - startTime));
		System.out.println(qsatResult.getQsatDump());
	}
	
	

	private List<String> getMinimumComponents() {
		List<String> implementation = new ArrayList<String>();
		implementation.add("cMobilePhone");
		implementation.add("Transmitter");
		implementation.add("Receiver");
		implementation.add("Mice");
		implementation.add("Speaker");
		implementation.add("Keypad");
		implementation.add("Battery");
		implementation.add("UtilInterface");
		implementation.add("SettingInterface");
		implementation.add("OSApps");
		implementation.add("UtilAdapter");
		implementation.add("MessageInterface");
		implementation.add("MessageAdapter");
		implementation.add("RingTonesApps");
		implementation.add("AlarmApps");
		implementation.add("SettingAdapter");
		implementation.add("OSApps");
		return implementation;
	}

	private List<String> getMinimumFeatures() {
		List<String> specification = new ArrayList<String>();
		specification.add("MobilePhone");
		specification.add("UtilitFunctions");
		specification.add("Calls");
		specification.add("Messaging");
		specification.add("Alarmclock");
		specification.add("RingingTones");
		specification.add("Setting");
		specification.add("OS");
		return specification;
	}

}
