/*
o, 	This file is part of FaMaTS.

    FaMaTS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FaMaTS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with FaMaTS.  If not, see <http://www.gnu.org/licenses/>.

 */
package es.us.isa.Sat4jReasoner;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import es.us.isa.FAMA.Benchmarking.PerformanceResult;
import es.us.isa.FAMA.Exceptions.FAMAParameterException;
import es.us.isa.FAMA.Reasoner.Question;
import es.us.isa.FAMA.models.featureModel.Cardinality;
import es.us.isa.FAMA.models.featureModel.GenericFeature;
import es.us.isa.FAMA.models.featureModel.GenericRelation;
import es.us.isa.FAMA.models.variabilityModel.VariabilityElement;
import es.us.isa.FAMA.stagedConfigManager.Configuration;

public class Sat4jReasoner extends in.barc.iitb.cfdvs.SPLANE.Reasoner.SplaneModelReasoner {

	/**
	 * @uml.property name="variables"
	 * @uml.associationEnd qualifier="key:java.lang.Object java.lang.String"
	 */
	private Map<String, String> variables; // Variables<FeatureName,SATVarNumber>
	private Map<String, String> traceability;
	private String traceFormula;
	private String cnf_content;
	private Map<String, GenericFeature> featuresMap;
	//private String pathFile;
	private ArrayList<String> clauses; // Clauses
	private ArrayList<String> clausesFeatures; // Clauses
	private ArrayList<String> clausesComponets; // Clauses
	private ArrayList<String> clausesTace;
	private int numvar; // Number of variables
	private ArrayList<String> addedClauses;
	
	
	public ArrayList<String> getClausesFeatures() {
		return clausesFeatures;
	}

	public void setClausesFeatures(ArrayList<String> clausesFeatures) {
		this.clausesFeatures = clausesFeatures;
	}

	public ArrayList<String> getClausesTace() {
		return clausesTace;
	}

	public void setClausesTace(ArrayList<String> clausesTace) {
		this.clausesTace = clausesTace;
	}

	public ArrayList<String> getClausesComponets() {
		return clausesComponets;
	}

	public void setClausesComponets(ArrayList<String> clausesComponets) {
		this.clausesComponets = clausesComponets;
	}

	public ArrayList<String> getAddedClauses() {
		return addedClauses;
	}

	public void setAddedClauses(ArrayList<String> addedClauses) {
		this.addedClauses = addedClauses;
	}

	public void setClauses(ArrayList<String> clauses) {
		this.clauses = clauses;
	}

	public Map<String, GenericFeature> getFeaturesMap() {
		return featuresMap;
	}

	public void setFeaturesMap(Map<String, GenericFeature> featuresMap) {
		this.featuresMap = featuresMap;
	}

	public Map<String, String> getVariables() {
		return variables;
	}

	public void setVariables(Map<String, String> variables) {
		this.variables = variables;
	}

	public Sat4jReasoner() {
		reset();
	}

	protected void finalize() {
	}

	@Override
	public void reset() {
		variables = new HashMap<String, String>();
		featuresMap = new HashMap<String, GenericFeature>();
		clauses = new ArrayList<String>();
		clausesFeatures = new ArrayList<String>();
		clausesComponets = new ArrayList<String>();
		clausesTace = new ArrayList<String>();
		addedClauses = new ArrayList<String>();
		setTraceability(new HashMap<String, String>());
		numvar = 1;
	}


	public ArrayList<String> getClauses() {
		return clauses;
	}

	public String getTraceFormula() {
		return traceFormula;
	}

	public void setTraceFormula(String traceFormula) {
		this.traceFormula = traceFormula;
	}

	public String getCNFVar(String featurename) {
		return variables.get(featurename);
	}

	public String getCNFContent() {
		return this.cnf_content;
	}

	public InputStream getCNFInputStream(){
		return new ByteArrayInputStream(cnf_content.getBytes());
	}
	public String getCnf_content() {
		return cnf_content;
	}

	public void setCnf_content(String cnf_content) {
		this.cnf_content = cnf_content;
	}

	// Return the name of the feature associate to cnf_var
	public GenericFeature getFeature(String cnf_var) {
		String featureName = "";
		Iterator<Entry<String, String>> it = variables.entrySet().iterator();
		boolean found = false;
		while (it.hasNext() && !found) {
			Entry<String, String> e = it.next();
			if (e.getValue().equals(cnf_var)) {
				found = true;
				featureName = e.getKey();
			}
		}
		return featuresMap.get(featureName);
	}

	// Generate the CNF File (Input of Sat4j)
	public void createSAT() {

		cnf_content = "c CNF file\n";

		// We show as comments the variables's number
		Iterator<String> it = variables.keySet().iterator();
		while (it.hasNext()) {
			String varName = it.next();
			cnf_content += "c var " + variables.get(varName) + " = " + varName
					+ "\n";
		}

		// Start the problem
		cnf_content += "p cnf " + variables.size() + " " + clauses.size()
				+ "\n";

		// Clauses
		it = clauses.iterator();
		while (it.hasNext()) {
			cnf_content += (String) it.next() + "\n";
		}

		// End file
		cnf_content += "0";
//
//		// Create the .cnf file
//		File outputFile = null;
//		try {
//
//			outputFile = File.createTempFile("cnf", "txt");// new
//			// File(filepath);
//			pathFile = outputFile.getAbsolutePath();
//			FileWriter out;
//			out = new FileWriter(outputFile);
//			out.write(cnf_content);
//			out.close();
//
//		} catch (IOException e) {
//			System.out.println("SAT: Error creating temporary cnf file");
//		}
	}

	@Override
	public void addCardinality(GenericRelation rel, GenericFeature child,
			GenericFeature parent, Iterator<Cardinality> cardinalities) {
		// TODO Cardinalities are not supported by SAT4j solver
	}

	@Override
	public void addExcludes(GenericRelation rel, GenericFeature origin,
			GenericFeature destination) {

		// Get features
		String cnf_origin = variables.get(origin.getName());
		String cnf_destination = variables.get(destination.getName());

		// Clause
		String cnf_excludes = "-" + cnf_origin + " -" + cnf_destination + " 0";
		clauses.add(cnf_excludes);
		
		if(origin.getName().charAt(0)=='c'){
			clausesComponets.add(cnf_excludes);
		}else{
			clausesFeatures.add(cnf_excludes);
		}
	}

	@Override
	public void addFeature(GenericFeature feature) {
		variables.put(feature.getName(), Integer.toString(numvar));
		numvar++;
		featuresMap.put(feature.getName(), feature);
	}

	@Override
	public void addMandatory(GenericRelation rel, GenericFeature child,
			GenericFeature parent) {
		// Get features
		String cnf_parent = variables.get(parent.getName());
		String cnf_child = variables.get(child.getName());

		// Clauses
		String cnf_mandatory1 = "-" + cnf_parent + " " + cnf_child + " 0";
		String cnf_mandatory2 = "-" + cnf_child + " " + cnf_parent + " 0";
		clauses.add(cnf_mandatory1);
		clauses.add(cnf_mandatory2);
		if(parent.getName().charAt(0)=='c'){
			clausesComponets.add(cnf_mandatory1);
			clausesComponets.add(cnf_mandatory2);
		}else{
			clausesFeatures.add(cnf_mandatory1);
			clausesFeatures.add(cnf_mandatory2);
		}
	}

	@Override
	public void addOptional(GenericRelation rel, GenericFeature child,
			GenericFeature parent) {
		// Get features
		String cnf_parent = variables.get(parent.getName());
		String cnf_child = variables.get(child.getName());

		// Clause
		String cnf_optional = "-" + cnf_child + " " + cnf_parent + " 0";
		clauses.add(cnf_optional);
		
		if(parent.getName().charAt(0)=='c'){
			clausesComponets.add(cnf_optional);
		}else{
			clausesFeatures.add(cnf_optional);
		}

	}

	@Override
	public void addRequires(GenericRelation rel, GenericFeature origin,
			GenericFeature destination) {

		// Get features
		String cnf_origin = variables.get(origin.getName());
		String cnf_destination = variables.get(destination.getName());

		// Clause
		String cnf_requires = "-" + cnf_origin + " " + cnf_destination + " 0";
		clauses.add(cnf_requires);
		if(origin.getName().charAt(0)=='c'){
			clausesComponets.add(cnf_requires);
		}else{
			clausesFeatures.add(cnf_requires);
		}
	}

	@Override
	public void addRoot(GenericFeature feature) {
		String root = variables.get(feature.getName());

		// Clause
		String cnf_root = root + " 0";
		clauses.add(cnf_root);
		if(feature.getName().charAt(0)=='c'){
			clausesComponets.add(cnf_root);
		}else{
			clausesFeatures.add(cnf_root);
		}
	}

	@Override
	public void addSet(GenericRelation rel, GenericFeature parent,
			Collection<GenericFeature> children,
			Collection<Cardinality> cardinalities) {
		
		GenericFeature feature=null;
		Iterator<Cardinality> iter = cardinalities.iterator();
		Cardinality card = iter.next();

		if (card.getMax() != 1) {

			// =============
			// OR Relation
			// =============

			String cnf_parent = variables.get(parent.getName());

			// (no parent or child1 or child2 ...or childn)

			String cnf_or = "-" + cnf_parent + " ";
			Iterator<GenericFeature> it = children.iterator();
			while (it.hasNext()) {
				feature = it.next();
				String cnf_child = variables.get(feature.getName());
				cnf_or += cnf_child + " ";
			}

			cnf_or += "0";
			clauses.add(cnf_or);
			if(feature.getName().charAt(0)=='c'){
				clausesComponets.add(cnf_or);
			}else{
				clausesFeatures.add(cnf_or);
			}
			

			// (no child1 or parent) and (no child2 or parent) and ... (no
			// childn or parent)

			it = children.iterator();
			while (it.hasNext()) {
				feature = (GenericFeature) it.next();
				String cnf_child = variables.get(feature.getName());
				cnf_or = "-" + cnf_child + " " + cnf_parent + " 0";
				clauses.add(cnf_or);
				if(feature.getName().charAt(0)=='c'){
					clausesComponets.add(cnf_or);
				}else{
					clausesFeatures.add(cnf_or);
				}
			}
		} else {

			// ======================
			// ALTERNATIVE Relation
			// ======================

			String cnf_parent = variables.get(parent.getName());

			// (children1 or children2 or ... or childrenN or no parent)

			String cnf_alternative = "-" + cnf_parent + " ";

			Iterator<GenericFeature> it = children.iterator();
			while (it.hasNext()) {
				feature = it.next();
				String cnf_child = variables.get(feature.getName());
				cnf_alternative += cnf_child + " ";
			}

			cnf_alternative += "0";
			clauses.add(cnf_alternative);
			if(feature.getName().charAt(0)=='c'){
				clausesComponets.add(cnf_alternative);
			}else{
				clausesFeatures.add(cnf_alternative);
			}

			// (no child1 or no child2) and ... (no child1 or no childN) and (no
			// child1 or parent) and
			// ...and (no childN or no childN-1) and (no childN or parent)

			// Insert the children collection in an ArrayList (We need direct
			// access with indexes)
			ArrayList<String> childrenList = new ArrayList<String>();
			it = children.iterator();
			while (it.hasNext()) {
				feature = (GenericFeature) it.next();
				String cnf_child = variables.get(feature.getName());
				childrenList.add(cnf_child);
			}

			for (int i = 0; i < childrenList.size(); i++) // for (int i = 0; i <
			// childrenList.size();
			// i++)
			{
				for (int k = i + 1; k < children.size(); k++) // for (int k = 0;
				// k <
				// childrenList.size();
				// k++)
				{
					if (i != k) {
						cnf_alternative = "-" + childrenList.get(i) + " -"
								+ childrenList.get(k) + " 0";
						clauses.add(cnf_alternative);
						if(feature.getName().charAt(0)=='c'){
							clausesComponets.add(cnf_alternative);
						}else{
							clausesFeatures.add(cnf_alternative);
						}
					}
				}

				cnf_alternative = "-" + childrenList.get(i) + " " + cnf_parent
						+ " 0";
				clauses.add(cnf_alternative);
				if(feature.getName().charAt(0)=='c'){
					clausesComponets.add(cnf_alternative);
				}else{
					clausesFeatures.add(cnf_alternative);
				}
			}
		}

	}

	public PerformanceResult ask(Question q) {
		if (q == null) {
			throw new FAMAParameterException("questions :Not specified");
		}
		
		PerformanceResult res=new Sat4jResult();
		
		Sat4jQuestion sq = (Sat4jQuestion) q;
		sq.preAnswer(this);
		res = sq.answer(this);
		sq.postAnswer(this);

		return res;
	}

	public Collection<GenericFeature> getAllFeatures() {
		return this.featuresMap.values();
	}

	public int getnumVar() {
		return numvar;
	}

	@Override
	public void applyStagedConfiguration(Configuration conf) {
		// Added Features
		Iterator<Entry<VariabilityElement, Integer>> it = conf.getElements()
				.entrySet().iterator();
		while (it.hasNext()) {
			Entry<VariabilityElement, Integer> entry = it.next();
			if (entry.getKey() instanceof GenericFeature) {
				String cnf_var = getCNFVar(entry.getKey().getName());
				String clause = "";
				if (entry.getValue() == 0) {
					clause = "-"+cnf_var + " 0";
				}
				if (entry.getValue() > 0) {
					clause = cnf_var + " 0";
					clauses.add(clause);
					if(entry.getKey().getName().charAt(0)=='c'){
						clausesComponets.add(clause);
					}else{
						clausesFeatures.add(clause);
					}
					addedClauses.add(clause);
				}
				clauses.add(clause);
				if(entry.getKey().getName().charAt(0)=='c'){
					clausesComponets.add(clause);
				}else{
					clausesFeatures.add(clause);
				}
				addedClauses.add(clause);
			}
		}

	}

	@Override
	public void unapplyStagedConfigurations() {
		Iterator<String> it = addedClauses.iterator();
		while (it.hasNext())
			this.getClauses().remove(it.next());
	}

	@Override
	public Map<String, Object> getHeusistics() {
		return new HashMap<String, Object>();
	}

	@Override
	public void setHeuristic(Object obj) {
		// Does not apply
	}

	
	@Override
	public String generateComponentConstraints() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public String generateFeatureConstraints() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public String generateTraceabilityFormula() {
		Iterator<Map.Entry<String, String>> entries = traceability.entrySet().iterator();
		String traceabilityFormula = new String();
		while (entries.hasNext()) {
			Map.Entry<String, String> entry = entries.next();
			String clauseTraceabililty = null;
			traceabilityFormula += "-" + variables.get(entry.getKey()) + " " + variables.get(entry.getValue()) + " 0"+ "\n";
			clauseTraceabililty = "-" + variables.get(entry.getKey()) + " " + variables.get(entry.getValue()) + " 0"+ "\n";
			clausesTace.add(clauseTraceabililty);
		}
		this.traceFormula=traceabilityFormula;
		return traceabilityFormula;
	}

	@Override
	public void addTraceability(Map<String, List<List<String>>> trace) {
		Iterator<Map.Entry<String, List<List<String>>>> entries = trace
				.entrySet().iterator();
		while (entries.hasNext()) {
			Map.Entry<String, List<List<String>>> entry = entries.next();
			String featureName = entry.getKey();
			String componentName=null;
			List<List<String>> allCompList = entry.getValue();
			Iterator<List<String>> allCompListItr = allCompList.iterator();
			while (allCompListItr.hasNext()) {
				List<String> compList = allCompListItr.next();
				Iterator<String> compListItr = compList.iterator();
				while (compListItr.hasNext()) {
					componentName = compListItr.next();
				}
			}
			traceability.put(featureName, componentName);
		}		
	}

	public Map<String, String> getTraceability() {
		return traceability;
	}

	public void setTraceability(Map<String, String> traceability) {
		this.traceability = traceability;
	}


}
