/*
	This file is part of FaMaTS.

    FaMaTS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FaMaTS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with FaMaTS.  If not, see <http://www.gnu.org/licenses/>.

 */
package es.us.isa.Sat4jReasoner.questions;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import org.sat4j.minisat.SolverFactory;
import org.sat4j.reader.DimacsReader;
import org.sat4j.reader.ParseFormatException;
import org.sat4j.reader.Reader;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.IProblem;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.TimeoutException;
import org.sat4j.tools.ModelIterator;

import es.us.isa.FAMA.Benchmarking.PerformanceResult;
import es.us.isa.FAMA.Exceptions.FAMAParameterException;
import es.us.isa.FAMA.Reasoner.Reasoner;
import es.us.isa.FAMA.Reasoner.questions.ValidQuestion;
import es.us.isa.Sat4jReasoner.Sat4jQuestion;
import es.us.isa.Sat4jReasoner.Sat4jReasoner;
import es.us.isa.Sat4jReasoner.Sat4jResult;

public class Sat4jImplementationQuestion extends Sat4jQuestion implements
		ValidQuestion {
	/**
	 * @uml.property name="valid"
	 */
	private boolean valid;
	private Collection<String> componentList;
	private String featureI;

	public Sat4jImplementationQuestion() {
		componentList = new ArrayList<String>();
		featureI = new String();
		valid = false;
	}

	public Sat4jImplementationQuestion(Collection<String> componentL,
			String featureI) {
		this.componentList = componentL;
		this.featureI = featureI;
		valid = false;
	}

	/**
	 * @return
	 * @uml.property name="valid"
	 */
	public boolean isValid() {
		return valid;
	}

	// Answer the question
	public PerformanceResult answer(Reasoner r) {
		if (r == null) {
			throw new FAMAParameterException("Reasoner :Not specified");
		}

		Sat4jResult res = new Sat4jResult();
		try {
		InputStream cnf = ((Sat4jReasoner)r).getCNFInputStream();
		ISolver solver = SolverFactory.instance().defaultSolver();
        ModelIterator mi = new ModelIterator(solver);
        solver.setTimeout(1800); // 1 hour timeout
        Reader reader = new DimacsReader(mi);
        Sat4jReasoner sr=(Sat4jReasoner)r;
		res.setClauses(sr.getClauses().size());
		res.setVariables((sr.getVariables().size()));
			res.setClauses(sr.getClauses().size());
			res.setVariables(sr.getVariables().size());
			IProblem problem = reader.parseInstance(cnf);
			while (problem.isSatisfiable()) {
				this.valid=true;
				break;
			}

		} catch (ParseFormatException e) {
			System.out.println("SatValidQuestion : Parse error "
					+ e.getMessage() + ". Check the sintax, please");
			res = null;
		} catch (IOException e) {
			System.out.println("SatValidQuestion : IOException: "
					+ e.getMessage());
			res = null;
		} catch (ContradictionException e) {
			// System.out.println("SatValidQuestion : UnSatisfiable (trivial)!");
			res = new Sat4jResult();
		} catch (TimeoutException e) {
			System.out.println("SatValidQuestion : Timeout, sorry!");
			res = null;
		}			
		return res;
	}

	public String toString() {
		if (valid)
			return "Feature is implementable by the given Implementation";
		else
			return "Feature is not implementable by the given Implementation";
	}

	@Override
	public void preAnswer(Reasoner r) {
		createSAT((Sat4jReasoner) r);

	}


	// Generate the CNF File (Input of Sat4j)
	public void createSAT(Sat4jReasoner r) {
		if(this.featureI==null || (this.featureI!=null && this.featureI.equals("")))
			this.featureI="root";
		Map<String, String> variables = r.getVariables();
		ArrayList<String> clausesComponent = r.getClausesComponets();
		String cnf_content = "c CNF file\n";
		int varCount = 0;
		// We show as comments the variables's number
		Iterator<String> it = variables.keySet().iterator();
		while (it.hasNext()) {
			String varName = it.next();
			if (varName.charAt(0) == 'c'){
				cnf_content += "c var " + variables.get(varName) + " = "
						+ varName + "\n";
				varCount++;
			}
		}
		// Start the problem
		cnf_content += "p cnf " + varCount + " "+ (clausesComponent.size()+1) + "\n";

		// Clauses
		it = clausesComponent.iterator();
		while (it.hasNext()) {
			cnf_content += (String) it.next() + "\n";
		}
		
		cnf_content += variables.get(r.getTraceability().get(this.featureI)) + " 0 \n";

		cnf_content += "0";
		r.setCnf_content(cnf_content);
		//System.out.println(cnf_content);
	}

}
