/*
	This file is part of FaMaTS.

    FaMaTS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FaMaTS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with FaMaTS.  If not, see <http://www.gnu.org/licenses/>.

 */
package es.us.isa.Sat4jReasoner.questions;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import org.sat4j.minisat.SolverFactory;
import org.sat4j.reader.DimacsReader;
import org.sat4j.reader.ParseFormatException;
import org.sat4j.reader.Reader;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.IProblem;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.TimeoutException;
import org.sat4j.tools.ModelIterator;

import es.us.isa.FAMA.Benchmarking.PerformanceResult;
import es.us.isa.FAMA.Exceptions.FAMAParameterException;
import es.us.isa.FAMA.Reasoner.Reasoner;
import es.us.isa.FAMA.Reasoner.questions.ValidQuestion;
import es.us.isa.Sat4jReasoner.Sat4jQuestion;
import es.us.isa.Sat4jReasoner.Sat4jReasoner;
import es.us.isa.Sat4jReasoner.Sat4jResult;

public class Sat4jIntersectionQuestion extends Sat4jQuestion implements
		ValidQuestion {
	/**
	 * @uml.property name="valid"
	 */
	private boolean valid;
	private Collection<String> listOne;
	private Collection<String> listTwo;
	private int[] listClauseOne;
	private int[] listClauseTwo;
	String type;

	public Sat4jIntersectionQuestion() {
		listOne = new ArrayList<String>();
		listTwo = new ArrayList<String>();
		valid = false;
	}

	public Sat4jIntersectionQuestion(Collection<String> listOne,	Collection<String> listwoT, String type) {
		this.listOne = listOne;
		this.listTwo = listTwo;
		this.type = type;
		valid = false;
	}

	/**
	 * @return
	 * @uml.property name="valid"
	 */
	public boolean isValid() {
		return valid;
	}

	// Answer the question
	public PerformanceResult answer(Reasoner r) {
		if (r == null) {
			throw new FAMAParameterException("Reasoner :Not specified");
		}
		Sat4jResult res = new Sat4jResult();
		try {
		InputStream cnf = ((Sat4jReasoner)r).getCNFInputStream();
		ISolver solver = SolverFactory.instance().defaultSolver();
        ModelIterator mi = new ModelIterator(solver);
        solver.setTimeout(1800); // 1 hour timeout
        Reader reader = new DimacsReader(mi);
        Sat4jReasoner sr=(Sat4jReasoner)r;
		res.setClauses(sr.getClauses().size());
		res.setVariables((sr.getVariables().size()));
		int flag=1;
			res.setClauses(sr.getClauses().size());
			res.setVariables(sr.getVariables().size());
			IProblem problem = reader.parseInstance(cnf);
			while (problem.isSatisfiable()) {
				int[] solution = problem.model();
				if(flag==1)
				{
					listClauseOne = solution;
					flag++;
				}else if (flag==2){
					listClauseTwo = solution;
					flag++;
				}else{
					break;
				}
			}
		} catch (ParseFormatException e) {
			System.out.println("SatValidQuestion : Parse error "
					+ e.getMessage() + ". Check the sintax, please");
			res = null;
		} catch (IOException e) {
			System.out.println("SatValidQuestion : IOException: "
					+ e.getMessage());
			res = null;
		} catch (ContradictionException e) {
			// System.out.println("SatValidQuestion : UnSatisfiable (trivial)!");
			res = new Sat4jResult();
		} catch (TimeoutException e) {
			System.out.println("SatValidQuestion : Timeout, sorry!");
			res = null;
		}					
		
		try {
			Sat4jReasoner sr=(Sat4jReasoner)r;
	        createSATForUnionImplementation(sr);
			InputStream cnf = ((Sat4jReasoner)r).getCNFInputStream();
			ISolver solver = SolverFactory.instance().defaultSolver();
	        ModelIterator mi = new ModelIterator(solver);
	        solver.setTimeout(3600); // 1 hour timeout
	        Reader reader = new DimacsReader(mi);
			res.setClauses(sr.getClauses().size());
			res.setVariables((sr.getVariables().size()));
				res.setClauses(sr.getClauses().size());
				res.setVariables(sr.getVariables().size());
				IProblem problem = reader.parseInstance(cnf);
				long before = System.currentTimeMillis();
				if (problem.isSatisfiable())
					valid = true;
				long time = System.currentTimeMillis() - before;
				res.setTime(time);
				res.fillFields(solver.getStat());
			} catch (ParseFormatException e) {
				System.out.println("SatValidQuestion : Parse error "
						+ e.getMessage() + ". Check the sintax, please");
				res = null;
			} catch (IOException e) {
				System.out.println("SatValidQuestion : IOException: "
						+ e.getMessage());
				res = null;
			} catch (ContradictionException e) {
				// System.out.println("SatValidQuestion : UnSatisfiable (trivial)!");
				res = new Sat4jResult();
			} catch (TimeoutException e) {
				System.out.println("SatValidQuestion : Timeout, sorry!");
				res = null;
			}
		return res;
	}

	public String toString() {
		if (valid)
			return "Intersection is possible";
		else
			return "Intersection is not possible";
	}

	@Override
	public void preAnswer(Reasoner r) {
		createSATForImplementation((Sat4jReasoner) r);

	}

	// Generate the CNF File (Input of Sat4j)
	public void createSATForImplementation(Sat4jReasoner r) {
		Map<String, String> variables = r.getVariables();
		ArrayList<String> clausesComponent = r.getClausesComponets();
		String cnf_content = "c CNF file\n";
		int varCount = 0;
		// We show as comments the variables's number
		Iterator<String> it = variables.keySet().iterator();
		while (it.hasNext()) {
			String varName = it.next();
			if (varName.charAt(0) == 'c')
				cnf_content += "c var " + variables.get(varName) + " = "
						+ varName + "\n";
			else
				varCount++;
		}
		// Start the problem
		cnf_content += "p cnf " + varCount + " "
				+ clausesComponent.size() + "\n";

		// Clauses
		it = clausesComponent.iterator();
		while (it.hasNext()) {
			cnf_content += (String) it.next() + "\n";
		}

		cnf_content += "0";
		r.setCnf_content(cnf_content);
		//System.out.println(cnf_content);
		//System.out.println("===========================");
	}
	
	
	// Generate the CNF File (Input of Sat4j)
	public void createSATForSpecification(Sat4jReasoner r) {
		Map<String, String> variables = r.getVariables();
		ArrayList<String> clausesFeatures = r.getClausesFeatures();
		String cnf_content = "c CNF file\n";
		int varCount = 0;
		// We show as comments the variables's number
		Iterator<String> it = variables.keySet().iterator();
		while (it.hasNext()) {
			String varName = it.next();
			if (varName.charAt(0) != 'c'){
				cnf_content += "c var " + variables.get(varName) + " = "
						+ varName + "\n";
				varCount++;
			}
		}
		// Start the problem
		cnf_content += "p cnf " + varCount + " "
				+ clausesFeatures.size() + "\n";

		// Clauses
		it = clausesFeatures.iterator();
		while (it.hasNext()) {
			cnf_content += (String) it.next() + "\n";
		}

		cnf_content += "0";
		r.setCnf_content(cnf_content);
		//System.out.println(cnf_content);
		//System.out.println("===========================");
	}
	
	// Generate the CNF File (Input of Sat4j)
	public void createSATForUnionImplementation(Sat4jReasoner r) {
		Map<String, String> variables = r.getVariables();
		ArrayList<String> clausesComponents = r.getClausesComponets();
		String cnf_content = "c CNF file\n";
		int varCount = 0;
		// We show as comments the variables's number
		Iterator<String> it = variables.keySet().iterator();
		while (it.hasNext()) {
			String varName = it.next();
			if (varName.charAt(0) == 'c'){
				cnf_content += "c var " + variables.get(varName) + " = "
						+ varName + "\n";
				varCount++;
			}
		}
		// Start the problem
		cnf_content += "p cnf " + varCount + " "+ (clausesComponents.size()+this.listClauseOne.length) + "\n";
		// Clauses
		it = clausesComponents.iterator();
		while (it.hasNext()) {
			cnf_content += (String) it.next() + "\n";
		}
		for(int i=0;i<this.listClauseOne.length;i++)
		{
			System.out.println("var temp: "+listClauseOne[i]+" \t "+listClauseTwo[i]);
			if(this.listClauseOne[i]>0 && this.listClauseTwo[i]>0)
			{
				cnf_content += Integer.toString(this.listClauseOne[i]) + " 0 \n";
			}else if(this.listClauseTwo[i]<0){
				cnf_content += Integer.toString(this.listClauseTwo[i]) + " 0 \n";
			}else{
				cnf_content += Integer.toString(this.listClauseOne[i]) + " 0 \n";
			}
		}

		cnf_content += "0";
		r.setCnf_content(cnf_content);
		System.out.println(cnf_content);
	}

}
