import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.Random;

import es.us.isa.ChocoReasoner.ChocoReasoner;
import es.us.isa.ChocoReasoner.questions.ChocoValidQuestion;
import es.us.isa.FAMA.models.FAMAfeatureModel.FAMAFeatureModel;
import es.us.isa.FAMA.models.FAMAfeatureModel.Feature;
import es.us.isa.FAMA.models.FAMAfeatureModel.fileformats.XMLWriter;
import es.us.isa.FAMA.models.variabilityModel.VariabilityModel;
import es.us.isa.generator.FM.AbstractFMGenerator;
import es.us.isa.generator.FM.FMGenerator;
import es.us.isa.generator.FM.GeneratorCharacteristics;

public class RamdomAttGenerator {

	public static void main(String[] args) throws Exception {

		PrintWriter out = new PrintWriter("./models/seeds.txt");
		int[] features = { 10, 20, 30, 50, 100, 200, 300, 400, 500,1000,3000,5000};
		int[] ctc = { 5, 10, 15 };
		Random r = new Random();
		for (int feat : features) {
			for (int cons : ctc) {
				for (int i = 0; i < 10; i++) {
					String name=feat+"-"+cons+"-"+i;

					GeneratorCharacteristics characteristics = new GeneratorCharacteristics();
					characteristics.setNumberOfFeatures(feat);		// Number of features
					characteristics.setPercentageCTC(cons);			// Percentage of cross-tree constraints.

					
					boolean valid=false;
					long seed=0;
					//int not=0;
					FAMAFeatureModel fm=null;
					while(!valid){
						seed=r.nextLong();
						characteristics.setSeed(seed);
						AbstractFMGenerator gen = new FMGenerator();
						AbstractFMGenerator generator = new FMGenerator();
						fm = (FAMAFeatureModel) generator.generateFM(characteristics);
					
	
						ChocoReasoner reasoner = new ChocoReasoner();
						fm.transformTo(reasoner);
						ChocoValidQuestion att= new ChocoValidQuestion();
						reasoner.ask(att);
						valid=att.isValid();
						//not++;
						//System.out.println(not);
						
					}
					// STEP 3: Save the model
					XMLWriter writer = new XMLWriter();
					writer.writeFile("./models/"+name+".fm", fm);
					writeTraceability("./models/"+name+".tfm", fm);
					changeName(fm);
					writer.writeFile("./models/"+name+".cfm", fm);
					out.print(name+";"+seed+"\r\n");
					out.flush();
					
					
					
					
				}
				out.close();
				System.out.println("Done");
			}
		}
	}

	private static void writeTraceability(String string, FAMAFeatureModel fm) {

		try {
			PrintWriter out = new PrintWriter(string);
			for(Feature f:fm.getFeatures()){
				out.write(f.getName()+" => { { c"+f.getName()+" } };\r\n");
			}
			out.flush();
			out.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}

		
	}

	private static void changeName(FAMAFeatureModel fm) {
		for(Feature f: fm.getFeatures()){
			f.setName("c"+f.getName());
		}
	}
}
