import in.barc.iitb.cfdvs.QSATReasoner.QSATReasoner;
import in.barc.iitb.cfdvs.SPLANE.models.parser.TraceabilityParser;
import in.barc.iitb.cfdvs.SPLANEModel.transformations.SplaneModelTransform;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedList;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import es.us.isa.FAMA.models.FAMAfeatureModel.fileformats.XMLReader;
import es.us.isa.FAMA.models.variabilityModel.TraceabilityModel;
import es.us.isa.FAMA.models.variabilityModel.VariabilityModel;
import es.us.isa.Sat4jReasoner.Sat4jReasoner;

public class experiment {

	public static void main(String[] args) throws IOException, InterruptedException {
        ExecutorService executor = Executors.newFixedThreadPool(12);

        PrintWriter out=new PrintWriter(new FileWriter("output.csv"), true);
		int[] features = { 10, 20, 30, 50, 100, 200, 300, 400, 500,1000,3000,5000};
		int[] ctc = { 5, 10, 15 };
		
		for (int feat : features) {
			for (int cons : ctc) {
				Collection<Runnable> threads = new LinkedList<Runnable>();

				for (int i = 0; i < 10; i++) {
					String name = "models/"+feat + "-" + cons + "-" + i;

					String featureFileName = name + ".fm";
					String componentFileName = name + ".cfm";
					String traceFileName = name + ".tfm";

					XMLReader reader = new XMLReader();
					VariabilityModel fm = null;
					try {
						fm = reader.parseFile(featureFileName);
					} catch (Exception e) {
						e.printStackTrace();
					}

					reader = new XMLReader();
					VariabilityModel cm = null;
					try {
						cm = reader.parseFile(componentFileName);
					} catch (Exception e) {
						e.printStackTrace();
					}

					
					
					SplaneModelTransform t = new SplaneModelTransform();
					TraceabilityParser trace = new TraceabilityParser(traceFileName);
					trace.parse();
					TraceabilityModel traceModel = new TraceabilityModel();
					traceModel.setTraceabilityOrgMap(trace.getTraProvidesMap());

					QSATReasoner reasoner = new QSATReasoner();
					t.transform(fm, cm, traceModel, reasoner);
					
					
					threads.add(new MyThread("VALID", feat, cons, i, reasoner, fm,cm,out));					
					threads.add(new MyThread("LIVE", feat, cons, i, reasoner, fm,cm,out));
					threads.add(new MyThread("SOUNDNESS", feat, cons, i, reasoner, fm,cm,out));
					threads.add(new MyThread("COMPLETENESS", feat, cons, i, reasoner, fm,cm,out));
					threads.add(new MyThread("COMMON", feat, cons, i, reasoner, fm,cm,out));
					threads.add(new MyThread("NOTCRITICAL", feat, cons, i, reasoner, fm,cm,out));
//					threads.add(new MyThread("INTERSECTION", feat, cons, i, reasoner, fm,cm));
					threads.add(new MyThread("IMPLEMENTS", feat, cons, i, reasoner, fm,cm,out));
					threads.add(new MyThread("EXISTENTIALLY", feat, cons, i, reasoner, fm,cm,out));
					threads.add(new MyThread("CRITICAL", feat, cons, i, reasoner, fm,cm,out));
					threads.add(new MyThread("COVERS", feat, cons, i, reasoner, fm,cm,out));
					threads.add(new MyThread("VALIDP", feat, cons, i, reasoner, fm,cm,out));
					threads.add(new MyThread("VALID", feat, cons, i, reasoner, fm,cm,out));
					
					Sat4jReasoner reasoner2= new Sat4jReasoner();
					t.transform(fm, cm, traceModel, reasoner2);

					threads.add(new MySat4jThread("VALID", feat, cons, i, reasoner2, fm,cm,out));					
					threads.add(new MySat4jThread("LIVE", feat, cons, i, reasoner2, fm,cm,out));
					threads.add(new MySat4jThread("SOUNDNESS", feat, cons, i, reasoner2, fm,cm,out));
					threads.add(new MySat4jThread("COMPLETENESS", feat, cons, i, reasoner2, fm,cm,out));
					threads.add(new MySat4jThread("COMMON", feat, cons, i, reasoner2, fm,cm,out));
					threads.add(new MySat4jThread("NOTCRITICAL", feat, cons, i, reasoner2, fm,cm,out));
//					threads.add(new MyThread("INTERSECTION", feat, cons, i, reasoner, fm,cm));
					threads.add(new MySat4jThread("IMPLEMENTS", feat, cons, i, reasoner2, fm,cm,out));
					threads.add(new MySat4jThread("EXISTENTIALLY", feat, cons, i, reasoner2, fm,cm,out));
					threads.add(new MySat4jThread("CRITICAL", feat, cons, i, reasoner2, fm,cm,out));
					threads.add(new MySat4jThread("COVERS", feat, cons, i, reasoner2, fm,cm,out));
					threads.add(new MySat4jThread("VALIDP", feat, cons, i, reasoner2, fm,cm,out));
					threads.add(new MySat4jThread("VALID", feat, cons, i, reasoner2, fm,cm,out));
					
					for(Runnable th:threads){
						executor.submit(th);

					}
					
					
				}
				
				
			}
			
			
			
			
		}
		executor.shutdown();
		out.flush();
		out.close();

	}

	

}
